function [PRisk, PRoR, PWts] = naiveMV(ERet, ECov, NPts)
ERet = ERet(:);
NASSETS = length(ERet);
V0 = zeros(1, NASSETS);
V1 = ones(1, NASSETS);

% Find the maximum expected return 
MaxReturnWeights = linprog(-ERet, [], [], V1, 1, V0);
MaxReturn = MaxReturnWeights'*ERet;

% Find the minimum variance return
MinVarWeights = quadprog(ECov, V0, [], [], V1, 1, V0);
MinVarReturn = MinVarWeights'*ERet;
MinVarStd = sqrt(MinVarWeights' * ECov * MinVarWeights);
    
if MaxReturn > MinVarReturn 
	   RTarget = linspace(MinVarReturn, MaxReturn, NPts);
	   NumFrontPoints = NPts;
else
      RTarget = MaxReturn;
      NumFrontPoints = 1;
end
		
PRoR = zeros(NumFrontPoints, 1);
PRisk = zeros(NumFrontPoints, 1);
PWts = zeros(NumFrontPoints, NASSETS);
PRoR(1) = MinVarReturn;
PRisk(1) = MinVarStd; 
PWts(1,:) = MinVarWeights(:)';

VConstr = ERet';
A = [V1 ; VConstr ];
B = [1 ; 0];
for point = 2:NumFrontPoints
	B(2) = RTarget(point);
	Weights = quadprog(ECov, V0, [], [], A, B, V0);
	PRoR(point) = dot(Weights, ERet);
	PRisk(point) = sqrt(Weights'*ECov*Weights);
	PWts(point, :) = Weights(:)';
end
